#!/bin/sh

nerr=0
nbytes=F424000
saddr="0x0"
pattype="0x1"
numbits="0x"
errbits="0x"
currerr=0
doread=1
dowrite=1
LWFPGA_SLAVES_BASE=0xFF200000 # Location for LWFPGA Slaves
MEM_CHECK_OFFSET=0x00010000 # Offset for mem_check component

usage="fpga_ddr_test.sh [-w | -r write/read only] [-b <number of BYTES to write/read)] [-s <address to start writing/reading (ex: 0x0)] [-i <int> (number of errors to attempt to inject)] [-p <PATTERN TYPE>] [-l <LWFPGA Slaves base location(ex: 0xff200000)>] [-o <mem_check offset(ex: 0x10000)>]"

echo "______________ STARTING DDR TEST ______________"

while getopts "i:b:p:rwl:o:" opt; do
	case $opt in
		i)
			nerr=${OPTARG};;
		b)
			printf -v nbytes '%x' ${OPTARG};;
		p)
			pattype=${OPTARG};;
		w)
			doread=0
			echo "WRITE ONLY";;
		r)
			dowrite=0
			echo "READ ONLY";;
		l)
			LWFPGA_SLAVES_BASE=${OPTARG};;
		o)
			MEM_CHECK_OFFSET=${OPTARG};;
		s)
			saddr=${OPTARG};;
		*)
			echo $usage
			exit;;
	esac
done

echo -n "Pattern Type: "
case $pattype in
	0x1 | 1)
		echo "PRBS7";;
	0x2 | 2)
		echo "PRBS15";;
	0x4 | 4)
		echo "PRBS23";;
	0x8 | 8)
		echo "PRBS31";;
	0x10 | 10)
		echo "High Frequency (ex. 01010101)";;
	0x20 | 20)
		echo "Low Frequency (ex. 11110000)";;
	*)
		echo "INVALID"
		exit
esac

printf -v loc '0x%x' $(( $LWFPGA_SLAVES_BASE + $MEM_CHECK_OFFSET ))
printf "mem_check Location: 0x%x\n" $loc
printf "writing %d bytes to %x\n" "0x"$nbytes $saddr
printf "errors to inject: %d\n" $nerr
echo "________________________________________"
### Register location setup ###
printf -v gen_en '0x%x' $(( $loc + 0x8000 ))
printf -v gen_pat_sel '0x%x' $(( $loc + 0x8004 ))
printf -v gen_ij_err '0x%x' $(( $loc + 0x8008 ))

printf -v check_status '0x%x' $(( $loc + 0x4000 ))
printf -v check_pat_sel '0x%x' $(( $loc + 0x4004 ))
printf -v check_count_ctrl '0x%x' $(( $loc + 0x4008 ))
printf -v check_nbits_l '0x%x' $(( $loc + 0x400c ))
printf -v check_nbits_h '0x%x' $(( $loc + 0x4010 ))
printf -v check_nerr_l '0x%x' $(( $loc + 0x4014 ))
printf -v check_nerr_h '0x%x' $(( $loc + 0x4018 ))

printf -v wr_desc_addr '0x%x' $(( $loc + 0x0044 ))
printf -v wr_desc_len '0x%x' $(( $loc + 0x0048 ))
printf -v wr_desc_ctrl '0x%x' $(( $loc + 0x005c ))
printf -v wr_csr_status '0x%x' $(( $loc + 0x0060 ))
printf -v wr_csr_ctrl '0x%x' $(( $loc + 0x0064 ))


printf -v rd_desc_addr '0x%x' $(( $loc + 0x0004 ))
printf -v rd_desc_len '0x%x' $(( $loc + 0x0008 ))
printf -v rd_desc_ctrl '0x%x' $(( $loc + 0x001c ))
printf -v rd_csr_status '0x%x' $(( $loc + 0x0020 ))
printf -v rd_csr_ctrl '0x%x' $(( $loc + 0x0024 ))

memtool $gen_pat_sel=$pattype
memtool $check_pat_sel=$pattype

if [ $dowrite -eq 1 ]; then
	memtool $gen_en=1
	memtool $wr_csr_ctrl=22
	
	echo "Creating descriptor for write (0x$nbytes Bytes)"
	memtool $wr_desc_addr=$saddr
	memtool $wr_desc_len=$nbytes
	memtool $wr_desc_ctrl=80000000

	echo "Waiting for write to finish and injecting $nerr errors" 
	while [  $( memtool $wr_csr_status 1 | tail -c10) != "00000002" ]; do
		if [ $currerr -ne $nerr ]; then
			memtool $gen_ij_err=1
			currerr=$(( $currerr+1 ))
		fi
	done

fi

if [ $doread -eq 1 ]; then
	memtool $check_status=1
	memtool $rd_csr_ctrl=22
	echo "Creating descriptor for read"

	memtool $rd_desc_addr=$saddr
	memtool $rd_desc_len=$nbytes
	memtool $rd_desc_ctrl=80000000

	echo "Waiting for read to finish"
	while [ $( memtool $rd_csr_status 1 | tail -c10) != "00000002" ]; do
	echo -n ""
	done



	echo "snapshotting counters"
	memtool $check_count_ctrl=1
	numbits+=$( memtool $check_nbits_h 1 | tail -c10)
	numbits+=$( memtool $check_nbits_l 1 | tail -c10)
	errbits+=$( memtool $check_nerr_h 1 | tail -c10)
	errbits+=$( memtool $check_nerr_l 1 | tail -c10)
fi

echo "Disabling/resetting generator/checker"
memtool $check_status=0
memtool $gen_en=0
memtool $check_count_ctrl=2

if [ $doread -eq 1 ]; then
	printf "Number of bytes written/read: %d/%d\n" 0x$nbytes $(( $numbits >> 3 ))
	printf "Number of errors injected/read: %d/%d:\n" $currerr $errbits
fi

nbytes=$(( 0x$nbytes ))
numbits=$(( ($numbits >> 3) + 656   ))
if [ $nbytes -eq $numbits  ] ; then 
	echo "Result: Passed"
else
	echo "Result: Failed"
fi

